@@ -10,21 +10,10 @@ import sys, StringIO, string, types, re
 from sqlalchemy import sql, engine, schema, ansisql, exceptions, pool, PassiveDefault
 import sqlalchemy.engine.default as default
 import sqlalchemy.types as sqltypes
-import datetime,time
-
-pysqlite2_timesupport = False   # Change this if the init.d guys ever get around to supporting time cols
-
-try:
-    from pysqlite2 import dbapi2 as sqlite
-except ImportError:
-    try:
-        from sqlite3 import dbapi2 as sqlite #try the 2.5+ stdlib name.
-    except ImportError:
-        try:
-            sqlite = __import__('sqlite') # skip ourselves
-        except:
-            sqlite = None
+import datetime,time, warnings
+import sqlalchemy.util as util
 
+    
 class SLNumeric(sqltypes.Numeric):
     def get_col_spec(self):
         if self.precision is None:
@@ -140,10 +129,6 @@ pragma_names = {
     'BLOB' : SLBinary,
 }
 
-if pysqlite2_timesupport:
-    colspecs.update({sqltypes.Time : SLTime})
-    pragma_names.update({'TIME' : SLTime})
-
 def descriptor():
     return {'name':'sqlite',
     'description':'SQLite',
@@ -152,38 +137,74 @@ def descriptor():
     ]}
 
 class SQLiteExecutionContext(default.DefaultExecutionContext):
-    def post_exec(self, engine, proxy, compiled, parameters, **kwargs):
-        if getattr(compiled, "isinsert", False):
-            self._last_inserted_ids = [proxy().lastrowid]
-
+    def post_exec(self):
+        if self.compiled.isinsert:
+            self._last_inserted_ids = [self.cursor.lastrowid]
+        super(SQLiteExecutionContext, self).post_exec()
+        
 class SQLiteDialect(ansisql.ANSIDialect):
+    
     def __init__(self, **kwargs):
+        ansisql.ANSIDialect.__init__(self, default_paramstyle='qmark', **kwargs)
         def vers(num):
             return tuple([int(x) for x in num.split('.')])
-        self.supports_cast = (sqlite is not None and vers(sqlite.sqlite_version) >= vers("3.2.3"))
-        ansisql.ANSIDialect.__init__(self, **kwargs)
+        self.supports_cast = (self.dbapi is None or vers(self.dbapi.sqlite_version) >= vers("3.2.3"))
+        if self.dbapi is not None:
+            sqlite_ver = self.dbapi.version_info
+            if sqlite_ver < (2,2) and sqlite_ver != (2,1,'3'):
+                warnings.warn(RuntimeWarning("The installed version of pysqlite2 is out-dated, and will cause errors in some cases.  Version 2.1.3 or greater is recommended."))
+            if vers(self.dbapi.sqlite_version) < vers("3.3.13"):
+                warnings.warn(RuntimeWarning("The installed version of sqlite is out-dated, and will cause errors in some cases.  Version 3.3.13 or greater is recommended."))
+        
+    def dbapi(cls):
+        try:
+            from pysqlite2 import dbapi2 as sqlite
+        except ImportError, e:
+            try:
+                from sqlite3 import dbapi2 as sqlite #try the 2.5+ stdlib name.
+            except ImportError:
+                try:
+                    sqlite = __import__('sqlite') # skip ourselves
+                except ImportError:
+                    raise e
+        return sqlite
+    dbapi = classmethod(dbapi)
 
     def compiler(self, statement, bindparams, **kwargs):
         return SQLiteCompiler(self, statement, bindparams, **kwargs)
 
     def schemagenerator(self, *args, **kwargs):
-        return SQLiteSchemaGenerator(*args, **kwargs)
+        return SQLiteSchemaGenerator(self, *args, **kwargs)
 
     def schemadropper(self, *args, **kwargs):
-        return SQLiteSchemaDropper(*args, **kwargs)
+        return SQLiteSchemaDropper(self, *args, **kwargs)
+
+    def supports_alter(self):
+        return False
 
     def preparer(self):
         return SQLiteIdentifierPreparer(self)
 
     def create_connect_args(self, url):
         filename = url.database or ':memory:'
-        return ([filename], url.query)
+
+        opts = url.query.copy()
+        util.coerce_kw_type(opts, 'timeout', float)
+        util.coerce_kw_type(opts, 'isolation_level', str)
+        util.coerce_kw_type(opts, 'detect_types', int)
+        util.coerce_kw_type(opts, 'check_same_thread', bool)
+        util.coerce_kw_type(opts, 'cached_statements', int)
+
+        return ([filename], opts)
 
     def type_descriptor(self, typeobj):
         return sqltypes.adapt_type(typeobj, colspecs)
 
-    def create_execution_context(self):
-        return SQLiteExecutionContext(self)
+    def create_execution_context(self, **kwargs):
+        return SQLiteExecutionContext(self, **kwargs)
+
+    def supports_unicode_statements(self):
+        return True
 
     def last_inserted_ids(self):
         return self.context.last_inserted_ids
@@ -191,9 +212,6 @@ class SQLiteDialect(ansisql.ANSIDialect):
     def oid_column_name(self, column):
         return "oid"
 
-    def dbapi(self):
-        return sqlite
-
     def has_table(self, connection, table_name, schema=None):
         cursor = connection.execute("PRAGMA table_info(" + table_name + ")", {})
         row = cursor.fetchone()
@@ -284,9 +302,6 @@ class SQLiteDialect(ansisql.ANSIDialect):
                     break
                 cols.append(row[2])
                 col = table.columns[row[2]]
-            # unique index that includes the pk is considered a multiple primary key
-            for col in cols:
-                table.primary_key.add(table.columns[col])
 
 class SQLiteCompiler(ansisql.ANSICompiler):
     def visit_cast(self, cast):
@@ -321,11 +336,9 @@ class SQLiteCompiler(ansisql.ANSICompiler):
             return ansisql.ANSICompiler.binary_operator_string(self, binary)
 
 class SQLiteSchemaGenerator(ansisql.ANSISchemaGenerator):
-    def supports_alter(self):
-        return False
 
     def get_column_specification(self, column, **kwargs):
-        colspec = self.preparer.format_column(column) + " " + column.type.engine_impl(self.engine).get_col_spec()
+        colspec = self.preparer.format_column(column) + " " + column.type.dialect_impl(self.dialect).get_col_spec()
         default = self.get_column_default_string(column)
         if default is not None:
             colspec += " DEFAULT " + default
@@ -345,12 +358,11 @@ class SQLiteSchemaGenerator(ansisql.ANSISchemaGenerator):
     #        super(SQLiteSchemaGenerator, self).visit_primary_key_constraint(constraint)
 
 class SQLiteSchemaDropper(ansisql.ANSISchemaDropper):
-    def supports_alter(self):
-        return False
+    pass
 
 class SQLiteIdentifierPreparer(ansisql.ANSIIdentifierPreparer):
     def __init__(self, dialect):
         super(SQLiteIdentifierPreparer, self).__init__(dialect, omit_schema=True)
 
 dialect = SQLiteDialect
-poolclass = pool.SingletonThreadPool
+dialect.poolclass = pool.SingletonThreadPool
