@@ -42,7 +42,7 @@ class RelationTest1(testbase.ORMTest):
         try:
             compile_mappers()
         except exceptions.ArgumentError, ar:
-            assert str(ar) == "Cant determine relation direction for relationship 'Person.manager (Manager)' - foreign key columns are present in both the parent and the child's mapped tables.  Specify 'foreign_keys' argument."
+            assert str(ar) == "Can't determine relation direction for relationship 'Person.manager (Manager)' - foreign key columns are present in both the parent and the child's mapped tables.  Specify 'foreign_keys' argument.", str(ar)
 
         clear_mappers()
 
@@ -247,9 +247,10 @@ class RelationTest3(testbase.ORMTest):
         sess = create_session()
         p = Person(name='person1')
         p2 = Person(name='person2')
+        p3 = Person(name='person3')
         m = Manager(name='manager1')
         p.colleagues.append(p2)
-        m.colleagues.append(p2)
+        m.colleagues.append(p3)
         if usedata:
             p.data = Data('ps data')
             m.data = Data('ms data')
@@ -261,9 +262,12 @@ class RelationTest3(testbase.ORMTest):
         sess.clear()
         p = sess.query(Person).get(p.person_id)
         p2 = sess.query(Person).get(p2.person_id)
-        print p, p2, p.colleagues
+        p3 = sess.query(Person).get(p3.person_id)
+        m = sess.query(Person).get(m.person_id)
+        print p, p2, p.colleagues, m.colleagues
         assert len(p.colleagues) == 1
         assert p.colleagues == [p2]
+        assert m.colleagues == [p3]
         if usedata:
             assert p.data.data == 'ps data'
             assert m.data.data == 'ms data'
@@ -773,6 +777,56 @@ class MultiLevelTest(testbase.ORMTest):
         assert set(session.query(Employee).select()) == set([a,b,c])
         assert set(session.query( Engineer).select()) == set([b,c])
         assert session.query( Manager).select() == [c]
+
+class ManyToManyPolyTest(testbase.ORMTest):
+    def define_tables(self, metadata):
+        global base_item_table, item_table, base_item_collection_table, collection_table
+        base_item_table = Table(
+            'base_item', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('child_name', String(255), default=None))
+
+        item_table = Table(
+            'item', metadata,
+            Column('id', Integer, ForeignKey('base_item.id'), primary_key=True),
+            Column('dummy', Integer, default=0)) # Dummy column to avoid weird insert problems
+
+        base_item_collection_table = Table(
+            'base_item_collection', metadata,
+            Column('item_id', Integer, ForeignKey('base_item.id')),
+            Column('collection_id', Integer, ForeignKey('collection.id')))
+
+        collection_table = Table(
+            'collection', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('name', Unicode(255)))
+            
+    def test_pjoin_compile(self):
+        """test that remote_side columns in the secondary join table arent attempted to be 
+        matched to the target polymorphic selectable"""
+        class BaseItem(object): pass
+        class Item(BaseItem): pass
+        class Collection(object): pass
+        item_join = polymorphic_union( {
+            'BaseItem':base_item_table.select(base_item_table.c.child_name=='BaseItem'),
+            'Item':base_item_table.join(item_table),
+            }, None, 'item_join')
+
+        mapper(
+            BaseItem, base_item_table,
+            select_table=item_join,
+            polymorphic_on=base_item_table.c.child_name,
+            polymorphic_identity='BaseItem',
+            properties=dict(collections=relation(Collection, secondary=base_item_collection_table, backref="items")))
+
+        mapper(
+            Item, item_table,
+            inherits=BaseItem,
+            polymorphic_identity='Item')
+
+        mapper(Collection, collection_table)
+        
+        class_mapper(BaseItem)
         
 if __name__ == "__main__":    
     testbase.main()
